@@ -61,6 +61,8 @@ typedef struct CLEANUP_STATE {
     char   *orig_rcpt;			/* original recipient address */
     char   *return_receipt;		/* return-receipt address */
     char   *errors_to;			/* errors-to address */
+    ARGV   *auto_hdrs;			/* MTA's own header(s) */
+    ARGV   *hbc_rcpt;			/* header/body checks BCC addresses */
     int     flags;			/* processing options, status flags */
     int     qmgr_opts;			/* qmgr processing options */
     int     errs;			/* any badness experienced */
@@ -114,7 +116,6 @@ typedef struct CLEANUP_STATE {
     VSTRING *milter_err_text;		/* milter call-back reply */
     HBC_CHECKS *milter_hbc_checks;	/* Milter header checks */
     VSTRING *milter_hbc_reply;		/* Milter header checks reply */
-    VSTRING *milter_orcpt_buf;		/* add_rcpt_par() orcpt */
 
     /*
      * Support for Milter body replacement requests.
@@ -213,10 +214,10 @@ extern void cleanup_all(void);
 extern void cleanup_sig(int);
 extern void cleanup_pre_jail(char *, char **);
 extern void cleanup_post_jail(char *, char **);
-extern CONFIG_INT_TABLE cleanup_int_table[];
-extern CONFIG_BOOL_TABLE cleanup_bool_table[];
-extern CONFIG_STR_TABLE cleanup_str_table[];
-extern CONFIG_TIME_TABLE cleanup_time_table[];
+extern const CONFIG_INT_TABLE cleanup_int_table[];
+extern const CONFIG_BOOL_TABLE cleanup_bool_table[];
+extern const CONFIG_STR_TABLE cleanup_str_table[];
+extern const CONFIG_TIME_TABLE cleanup_time_table[];
 
 #define CLEANUP_RECORD(s, t, b, l)	((s)->action((s), (t), (b), (l)))
 
