@@ -68,14 +68,11 @@
 #include <sys_defs.h>
 #include <string.h>
 
-#ifdef STRCASECMP_IN_STRINGS_H
-#include <strings.h>
-#endif
-
 /* Utility library. */
 
 #include <msg.h>
 #include <vstring.h>
+#include <stringops.h>
 
 /* Global library. */
 
@@ -105,8 +102,8 @@ int     verify_append(const char *queue_id, MSG_STATS *stats,
     if (var_verify_neg_cache || vrfy_stat == DEL_RCPT_STAT_OK) {
 	req_stat = verify_clnt_update(recipient->orig_addr, vrfy_stat,
 				      my_dsn.reason);
-	if (req_stat == VRFY_STAT_OK && strcasecmp(recipient->address,
-						 recipient->orig_addr) != 0)
+	if (req_stat == VRFY_STAT_OK
+	    && strcasecmp_utf8(recipient->address, recipient->orig_addr) != 0)
 	    req_stat = verify_clnt_update(recipient->address, vrfy_stat,
 					  my_dsn.reason);
     } else {
