@@ -43,63 +43,63 @@
 /*	The argv argument specifies command-line arguments left over
 /*	after options processing.
 /* .PP
-/*	Optional arguments are specified as a null-terminated (key, value)
-/*	list. Keys and expected values are:
-/* .IP "MAIL_SERVER_INT_TABLE (CONFIG_INT_TABLE *)"
+/*	Optional arguments are specified as a null-terminated list
+/*	with macros that have zero or more arguments:
+/* .IP "CA_MAIL_SERVER_INT_TABLE(CONFIG_INT_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_LONG_TABLE (CONFIG_LONG_TABLE *)"
+/* .IP "CA_MAIL_SERVER_LONG_TABLE(CONFIG_LONG_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_STR_TABLE (CONFIG_STR_TABLE *)"
+/* .IP "CA_MAIL_SERVER_STR_TABLE(CONFIG_STR_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_BOOL_TABLE (CONFIG_BOOL_TABLE *)"
+/* .IP "CA_MAIL_SERVER_BOOL_TABLE(CONFIG_BOOL_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_TIME_TABLE (CONFIG_TIME_TABLE *)"
+/* .IP "CA_MAIL_SERVER_TIME_TABLE(CONFIG_TIME_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_RAW_TABLE (CONFIG_RAW_TABLE *)"
+/* .IP "CA_MAIL_SERVER_RAW_TABLE(CONFIG_RAW_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed. Raw parameters are not subjected to $name
 /*	evaluation.
-/* .IP "MAIL_SERVER_NINT_TABLE (CONFIG_NINT_TABLE *)"
+/* .IP "CA_MAIL_SERVER_NINT_TABLE(CONFIG_NINT_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_NBOOL_TABLE (CONFIG_NBOOL_TABLE *)"
+/* .IP "CA_MAIL_SERVER_NBOOL_TABLE(CONFIG_NBOOL_TABLE *)"
 /*	A table with configurable parameters, to be loaded from the
 /*	global Postfix configuration file. Tables are loaded in the
 /*	order as specified, and multiple instances of the same type
 /*	are allowed.
-/* .IP "MAIL_SERVER_PRE_INIT (void *(char *service_name, char **argv))"
+/* .IP "CA_MAIL_SERVER_PRE_INIT(void *(char *service_name, char **argv))"
 /*	A pointer to a function that is called once
 /*	by the skeleton after it has read the global configuration file
 /*	and after it has processed command-line arguments, but before
 /*	the skeleton has optionally relinquished the process privileges.
 /* .sp
 /*	Only the last instance of this parameter type is remembered.
-/* .IP "MAIL_SERVER_POST_INIT (void *(char *service_name, char **argv))"
+/* .IP "CA_MAIL_SERVER_POST_INIT(void *(char *service_name, char **argv))"
 /*	A pointer to a function that is called once
 /*	by the skeleton after it has optionally relinquished the process
 /*	privileges, but before servicing client connection requests.
 /* .sp
 /*	Only the last instance of this parameter type is remembered.
-/* .IP "MAIL_SERVER_LOOP (int *(char *service_name, char **argv))"
+/* .IP "CA_MAIL_SERVER_LOOP(int *(char *service_name, char **argv))"
 /*	A pointer to function that is executed from
 /*	within the event loop, whenever an I/O or timer event has happened,
 /*	or whenever nothing has happened for a specified amount of time.
@@ -107,28 +107,28 @@
 /*	the next event. Specify -1 to wait for "as long as it takes".
 /* .sp
 /*	Only the last instance of this parameter type is remembered.
-/* .IP "MAIL_SERVER_EXIT (void *(char *service_name, char **argv))"
+/* .IP "CA_MAIL_SERVER_EXIT(void *(char *service_name, char **argv))"
 /*	A pointer to function that is executed immediately before normal
 /*	process termination.
-/* .IP "MAIL_SERVER_PRE_ACCEPT (void *(char *service_name, char **argv))"
+/* .IP "CA_MAIL_SERVER_PRE_ACCEPT(void *(char *service_name, char **argv))"
 /*	Function to be executed prior to accepting a new connection.
 /* .sp
 /*	Only the last instance of this parameter type is remembered.
-/* .IP "MAIL_SERVER_PRE_DISCONN (VSTREAM *, char *service_name, char **argv)"
+/* .IP "CA_MAIL_SERVER_PRE_DISCONN(VSTREAM *, char *service_name, char **argv)"
 /*	A pointer to a function that is called
 /*	by the multi_server_disconnect() function (see below).
 /* .sp
 /*	Only the last instance of this parameter type is remembered.
-/* .IP "MAIL_SERVER_IN_FLOW_DELAY (none)"
+/* .IP "CA_MAIL_SERVER_IN_FLOW_DELAY
 /*	Pause $in_flow_delay seconds when no "mail flow control token"
 /*	is available. A token is consumed for each connection request.
-/* .IP MAIL_SERVER_SOLITARY
+/* .IP CA_MAIL_SERVER_SOLITARY
 /*	This service must be configured with process limit of 1.
-/* .IP MAIL_SERVER_UNLIMITED
+/* .IP CA_MAIL_SERVER_UNLIMITED
 /*	This service must be configured with process limit of 0.
-/* .IP MAIL_SERVER_PRIVILEGED
+/* .IP CA_MAIL_SERVER_PRIVILEGED
 /*	This service must be configured as privileged.
-/* .IP "MAIL_SERVER_BOUNCE_INIT (const char *, const char **)"
+/* .IP "CA_MAIL_SERVER_BOUNCE_INIT(const char *, const char **)"
 /*	Initialize the DSN filter for the bounce/defer service
 /*	clients with the specified map source and map names.
 /* .PP
@@ -241,7 +241,7 @@ static int socket_count = 1;
 static void (*multi_server_service) (VSTREAM *, char *, char **);
 static char *multi_server_name;
 static char **multi_server_argv;
-static void (*multi_server_accept) (int, char *);
+static void (*multi_server_accept) (int, void *);
 static void (*multi_server_onexit) (char *, char **);
 static void (*multi_server_pre_accept) (char *, char **);
 static VSTREAM *multi_server_lock;
@@ -261,7 +261,7 @@ static NORETURN multi_server_exit(void)
 
 /* multi_server_abort - terminate after abnormal master exit */
 
-static void multi_server_abort(int unused_event, char *unused_context)
+static void multi_server_abort(int unused_event, void *unused_context)
 {
     if (msg_verbose)
 	msg_info("master disconnect -- exiting");
@@ -270,14 +270,14 @@ static void multi_server_abort(int unused_event, char *unused_context)
 
 /* multi_server_timeout - idle time exceeded */
 
-static void multi_server_timeout(int unused_event, char *unused_context)
+static void multi_server_timeout(int unused_event, void *unused_context)
 {
     if (msg_verbose)
 	msg_info("idle timeout -- exiting");
     multi_server_exit();
 }
 
-/*  multi_server_drain - stop accepting new clients */
+/* multi_server_drain - stop accepting new clients */
 
 int     multi_server_drain(void)
 {
@@ -322,12 +322,12 @@ void    multi_server_disconnect(VSTREAM *stream)
     if (use_count < INT_MAX)
 	use_count++;
     if (client_count == 0 && var_idle_limit > 0)
-	event_request_timer(multi_server_timeout, (char *) 0, var_idle_limit);
+	event_request_timer(multi_server_timeout, (void *) 0, var_idle_limit);
 }
 
 /* multi_server_execute - in case (char *) != (struct *) */
 
-static void multi_server_execute(int unused_event, char *context)
+static void multi_server_execute(int unused_event, void *context)
 {
     VSTREAM *stream = (VSTREAM *) context;
     HTABLE *attr = (vstream_flags(stream) == multi_server_saved_flags ?
@@ -358,11 +358,11 @@ static void multi_server_execute(int unused_event, char *context)
 
 /* multi_server_enable_read - enable read events */
 
-static void multi_server_enable_read(int unused_event, char *context)
+static void multi_server_enable_read(int unused_event, void *context)
 {
     VSTREAM *stream = (VSTREAM *) context;
 
-    event_enable_read(vstream_fileno(stream), multi_server_execute, (char *) stream);
+    event_enable_read(vstream_fileno(stream), multi_server_execute, (void *) stream);
 }
 
 /* multi_server_wakeup - wake up application */
@@ -397,24 +397,24 @@ static void multi_server_wakeup(int fd, HTABLE *attr)
     stream = vstream_fdopen(fd, O_RDWR);
     tmp = concatenate(multi_server_name, " socket", (char *) 0);
     vstream_control(stream,
-		    VSTREAM_CTL_PATH, tmp,
-		    VSTREAM_CTL_CONTEXT, (char *) attr,
-		    VSTREAM_CTL_END);
+		    CA_VSTREAM_CTL_PATH(tmp),
+		    CA_VSTREAM_CTL_CONTEXT((void *) attr),
+		    CA_VSTREAM_CTL_END);
     myfree(tmp);
     timed_ipc_setup(stream);
     multi_server_saved_flags = vstream_flags(stream);
     if (multi_server_in_flow_delay && mail_flow_get(1) < 0)
-	event_request_timer(multi_server_enable_read, (char *) stream,
+	event_request_timer(multi_server_enable_read, (void *) stream,
 			    var_in_flow_delay);
     else
-	multi_server_enable_read(0, (char *) stream);
+	multi_server_enable_read(0, (void *) stream);
 }
 
 /* multi_server_accept_local - accept client connection request */
 
-static void multi_server_accept_local(int unused_event, char *context)
+static void multi_server_accept_local(int unused_event, void *context)
 {
-    int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
+    int     listen_fd = CAST_ANY_PTR_TO_INT(context);
     int     time_left = -1;
     int     fd;
 
@@ -426,7 +426,7 @@ static void multi_server_accept_local(int unused_event, char *context)
      * minimize confusion.
      */
     if (client_count == 0 && var_idle_limit > 0)
-	time_left = event_cancel_timer(multi_server_timeout, (char *) 0);
+	time_left = event_cancel_timer(multi_server_timeout, (void *) 0);
 
     if (multi_server_pre_accept)
 	multi_server_pre_accept(multi_server_name, multi_server_argv);
@@ -439,7 +439,7 @@ static void multi_server_accept_local(int unused_event, char *context)
 	if (errno != EAGAIN)
 	    msg_error("accept connection: %m");
 	if (time_left >= 0)
-	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
+	    event_request_timer(multi_server_timeout, (void *) 0, time_left);
 	return;
     }
     multi_server_wakeup(fd, (HTABLE *) 0);
@@ -449,9 +449,9 @@ static void multi_server_accept_local(int unused_event, char *context)
 
 /* multi_server_accept_pass - accept descriptor */
 
-static void multi_server_accept_pass(int unused_event, char *context)
+static void multi_server_accept_pass(int unused_event, void *context)
 {
-    int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
+    int     listen_fd = CAST_ANY_PTR_TO_INT(context);
     int     time_left = -1;
     int     fd;
     HTABLE *attr = 0;
@@ -464,7 +464,7 @@ static void multi_server_accept_pass(int unused_event, char *context)
      * minimize confusion.
      */
     if (client_count == 0 && var_idle_limit > 0)
-	time_left = event_cancel_timer(multi_server_timeout, (char *) 0);
+	time_left = event_cancel_timer(multi_server_timeout, (void *) 0);
 
     if (multi_server_pre_accept)
 	multi_server_pre_accept(multi_server_name, multi_server_argv);
@@ -477,7 +477,7 @@ static void multi_server_accept_pass(int unused_event, char *context)
 	if (errno != EAGAIN)
 	    msg_error("accept connection: %m");
 	if (time_left >= 0)
-	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
+	    event_request_timer(multi_server_timeout, (void *) 0, time_left);
 	return;
     }
     multi_server_wakeup(fd, attr);
@@ -487,9 +487,9 @@ static void multi_server_accept_pass(int unused_event, char *context)
 
 /* multi_server_accept_inet - accept client connection request */
 
-static void multi_server_accept_inet(int unused_event, char *context)
+static void multi_server_accept_inet(int unused_event, void *context)
 {
-    int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
+    int     listen_fd = CAST_ANY_PTR_TO_INT(context);
     int     time_left = -1;
     int     fd;
 
@@ -501,7 +501,7 @@ static void multi_server_accept_inet(int unused_event, char *context)
      * minimize confusion.
      */
     if (client_count == 0 && var_idle_limit > 0)
-	time_left = event_cancel_timer(multi_server_timeout, (char *) 0);
+	time_left = event_cancel_timer(multi_server_timeout, (void *) 0);
 
     if (multi_server_pre_accept)
 	multi_server_pre_accept(multi_server_name, multi_server_argv);
@@ -514,7 +514,7 @@ static void multi_server_accept_inet(int unused_event, char *context)
 	if (errno != EAGAIN)
 	    msg_error("accept connection: %m");
 	if (time_left >= 0)
-	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
+	    event_request_timer(multi_server_timeout, (void *) 0, time_left);
 	return;
     }
     multi_server_wakeup(fd, (HTABLE *) 0);
@@ -875,9 +875,9 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
      */
     if (stream != 0) {
 	vstream_control(stream,
-			VSTREAM_CTL_DOUBLE,
-			VSTREAM_CTL_WRITE_FD, STDOUT_FILENO,
-			VSTREAM_CTL_END);
+			CA_VSTREAM_CTL_DOUBLE,
+			CA_VSTREAM_CTL_WRITE_FD(STDOUT_FILENO),
+			CA_VSTREAM_CTL_END);
 	service(stream, multi_server_name, multi_server_argv);
 	vstream_fflush(stream);
 	multi_server_exit();
@@ -890,16 +890,16 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
      * when the master process terminated abnormally.
      */
     if (var_idle_limit > 0)
-	event_request_timer(multi_server_timeout, (char *) 0, var_idle_limit);
+	event_request_timer(multi_server_timeout, (void *) 0, var_idle_limit);
     for (fd = MASTER_LISTEN_FD; fd < MASTER_LISTEN_FD + socket_count; fd++) {
-	event_enable_read(fd, multi_server_accept, CAST_INT_TO_CHAR_PTR(fd));
+	event_enable_read(fd, multi_server_accept, CAST_INT_TO_VOID_PTR(fd));
 	close_on_exec(fd, CLOSE_ON_EXEC);
     }
-    event_enable_read(MASTER_STATUS_FD, multi_server_abort, (char *) 0);
+    event_enable_read(MASTER_STATUS_FD, multi_server_abort, (void *) 0);
     close_on_exec(MASTER_STATUS_FD, CLOSE_ON_EXEC);
     close_on_exec(MASTER_FLOW_READ, CLOSE_ON_EXEC);
     close_on_exec(MASTER_FLOW_WRITE, CLOSE_ON_EXEC);
-    watchdog = watchdog_create(var_daemon_timeout, (WATCHDOG_FN) 0, (char *) 0);
+    watchdog = watchdog_create(var_daemon_timeout, (WATCHDOG_FN) 0, (void *) 0);
 
     /*
      * The event loop, at last.
