@@ -85,7 +85,7 @@
 #include <stringops.h>
 #include <myaddrinfo.h>
 #include <inet_proto.h>
-#include <midna.h>
+#include <midna_domain.h>
 
 /* Global library. */
 
@@ -142,8 +142,8 @@ static DNS_RR *smtp_addr_one(DNS_RR *addr_list, const char *host, int res_opt,
     /*
      * Interpret a numerical name as an address.
      */
-    if (hostaddr_to_sockaddr(host, (char *) 0, 0, &res0) == 0
-     && strchr((char *) proto_info->sa_family_list, res0->ai_family) != 0) {
+    if (hostaddr_to_sockaddr(host, (char *) 0, 0, &res0) == 0) {
+	if (strchr((char *) proto_info->sa_family_list, res0->ai_family) != 0) {
 	if ((addr = dns_sa_to_rr(host, pref, res0->ai_addr)) == 0)
 	    msg_fatal("host %s: conversion error for address family %d: %m",
 		    host, ((struct sockaddr *) (res0->ai_addr))->sa_family);
@@ -151,6 +151,8 @@ static DNS_RR *smtp_addr_one(DNS_RR *addr_list, const char *host, int res_opt,
 	freeaddrinfo(res0);
 	return (addr_list);
     }
+	freeaddrinfo(res0);
+    }
 
     /*
      * Use DNS lookup, but keep the option open to use native name service.
@@ -177,6 +179,9 @@ static DNS_RR *smtp_addr_one(DNS_RR *addr_list, const char *host, int res_opt,
 	case DNS_INVAL:
 	    dsb_status(why, SMTP_HAS_SOFT_DSN(why) ? "4.4.4" : "5.4.4");
 	    return (addr_list);
+	case DNS_POLICY:
+	    dsb_status(why, "4.7.0");
+	    return (addr_list);
 	case DNS_NOTFOUND:
 	    dsb_status(why, SMTP_HAS_SOFT_DSN(why) ? "4.4.4" : "5.4.4");
 	    /* maybe native naming service will succeed */
@@ -373,7 +378,7 @@ DNS_RR *smtp_domain_addr(const char *name, DNS_RR **mxrr, int misc_flags,
      * IDNA support.
      */
 #ifndef NO_EAI
-    if (!allascii(name) && (aname = midna_utf8_to_ascii(name)) != 0) {
+    if (!allascii(name) && (aname = midna_domain_to_ascii(name)) != 0) {
 	if (msg_verbose)
 	    msg_info("%s asciified to %s", name, aname);
     } else
@@ -433,6 +438,12 @@ DNS_RR *smtp_domain_addr(const char *name, DNS_RR **mxrr, int misc_flags,
 	if (var_ign_mx_lookup_err)
 	    addr_list = smtp_host_addr(aname, misc_flags, why);
 	break;
+    case DNS_NULLMX:
+	dsb_status(why, "5.1.0");
+	break;
+    case DNS_POLICY:
+	dsb_status(why, "4.7.0");
+	break;
     case DNS_FAIL:
 	dsb_status(why, "5.4.3");
 	if (var_ign_mx_lookup_err)
@@ -513,7 +524,7 @@ DNS_RR *smtp_host_addr(const char *host, int misc_flags, DSN_BUF *why)
      * IDNA support.
      */
 #ifndef NO_EAI
-    if (!allascii(host) && (ahost = midna_utf8_to_ascii(host)) != 0) {
+    if (!allascii(host) && (ahost = midna_domain_to_ascii(host)) != 0) {
 	if (msg_verbose)
 	    msg_info("%s asciified to %s", host, ahost);
     } else
