@@ -151,7 +151,7 @@
 /* SMTPUTF8 CONTROLS
 /* .ad
 /* .fi
-/*	Preliminary SMTPUTF8 support is introduced with Postfix 2.12.
+/*	Preliminary SMTPUTF8 support is introduced with Postfix 3.0.
 /* .IP "\fBsmtputf8_autodetect_classes (sendmail, verify)\fR"
 /*	Detect that a message requires SMTPUTF8 support for the specified
 /*	mail origin classes.
@@ -359,9 +359,9 @@ static void verify_update_service(VSTREAM *client_stream)
     long    updated;
 
     if (attr_scan(client_stream, ATTR_FLAG_STRICT,
-		  ATTR_TYPE_STR, MAIL_ATTR_ADDR, addr,
-		  ATTR_TYPE_INT, MAIL_ATTR_ADDR_STATUS, &addr_status,
-		  ATTR_TYPE_STR, MAIL_ATTR_WHY, text,
+		  RECV_ATTR_STR(MAIL_ATTR_ADDR, addr),
+		  RECV_ATTR_INT(MAIL_ATTR_ADDR_STATUS, &addr_status),
+		  RECV_ATTR_STR(MAIL_ATTR_WHY, text),
 		  ATTR_TYPE_END) == 3) {
 	/* FIX 200501 IPv6 patch did not neuter ":" in address literals. */
 	translit(STR(addr), ":", "_");
@@ -369,7 +369,7 @@ static void verify_update_service(VSTREAM *client_stream)
 	    msg_warn("bad recipient status %d for recipient %s",
 		     addr_status, STR(addr));
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
+		       SEND_ATTR_INT(MAIL_ATTR_STATUS, VRFY_STAT_BAD),
 		       ATTR_TYPE_END);
 	} else {
 
@@ -391,7 +391,7 @@ static void verify_update_service(VSTREAM *client_stream)
 		dict_cache_update(verify_map, STR(addr), STR(buf));
 	    }
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_OK,
+		       SEND_ATTR_INT(MAIL_ATTR_STATUS, VRFY_STAT_OK),
 		       ATTR_TYPE_END);
 	}
     }
@@ -427,7 +427,7 @@ static void verify_query_service(VSTREAM *client_stream)
     char   *text;
 
     if (attr_scan(client_stream, ATTR_FLAG_STRICT,
-		  ATTR_TYPE_STR, MAIL_ATTR_ADDR, addr,
+		  RECV_ATTR_STR(MAIL_ATTR_ADDR, addr),
 		  ATTR_TYPE_END) == 1) {
 	long    now = (long) time((time_t *) 0);
 
@@ -478,9 +478,9 @@ static void verify_query_service(VSTREAM *client_stream)
 	 * Respond to the client.
 	 */
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_OK,
-		   ATTR_TYPE_INT, MAIL_ATTR_ADDR_STATUS, addr_status,
-		   ATTR_TYPE_STR, MAIL_ATTR_WHY, text,
+		   SEND_ATTR_INT(MAIL_ATTR_STATUS, VRFY_STAT_OK),
+		   SEND_ATTR_INT(MAIL_ATTR_ADDR_STATUS, addr_status),
+		   SEND_ATTR_STR(MAIL_ATTR_WHY, text),
 		   ATTR_TYPE_END);
 
 	/*
@@ -532,7 +532,7 @@ static void verify_query_service(VSTREAM *client_stream)
 /* verify_cache_validator - cache cleanup validator */
 
 static int verify_cache_validator(const char *addr, const char *raw_data,
-				          char *context)
+				          void *context)
 {
     VSTRING *get_buf = (VSTRING *) context;
     int     addr_status;
@@ -572,7 +572,7 @@ static void verify_service(VSTREAM *client_stream, char *unused_service,
      */
     if (attr_scan(client_stream,
 		  ATTR_FLAG_MORE | ATTR_FLAG_STRICT,
-		  ATTR_TYPE_STR, MAIL_ATTR_REQ, request,
+		  RECV_ATTR_STR(MAIL_ATTR_REQ, request),
 		  ATTR_TYPE_END) == 1) {
 	if (STREQ(STR(request), VRFY_REQ_UPDATE)) {
 	    verify_update_service(client_stream);
@@ -581,7 +581,7 @@ static void verify_service(VSTREAM *client_stream, char *unused_service,
 	} else {
 	    msg_warn("unrecognized request: \"%s\", ignored", STR(request));
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
+		       SEND_ATTR_INT(MAIL_ATTR_STATUS, VRFY_STAT_BAD),
 		       ATTR_TYPE_END);
 	}
     }
@@ -591,7 +591,7 @@ static void verify_service(VSTREAM *client_stream, char *unused_service,
 
 /* verify_dump - dump some statistics */
 
-static void verify_dump(void)
+static void verify_dump(char *unused_name, char **unused_argv)
 {
 
     /*
@@ -629,11 +629,11 @@ static void post_jail_init(char *unused_name, char **unused_argv)
 	if (msg_verbose)
 	    cache_flags |= DICT_CACHE_FLAG_VERBOSE;
 	dict_cache_control(verify_map,
-			   DICT_CACHE_CTL_FLAGS, cache_flags,
-			   DICT_CACHE_CTL_INTERVAL, var_verify_scan_cache,
-			   DICT_CACHE_CTL_VALIDATOR, verify_cache_validator,
-			DICT_CACHE_CTL_CONTEXT, (char *) vstring_alloc(100),
-			   DICT_CACHE_CTL_END);
+			   CA_DICT_CACHE_CTL_FLAGS(cache_flags),
+			   CA_DICT_CACHE_CTL_INTERVAL(var_verify_scan_cache),
+			CA_DICT_CACHE_CTL_VALIDATOR(verify_cache_validator),
+		     CA_DICT_CACHE_CTL_CONTEXT((void *) vstring_alloc(100)),
+			   CA_DICT_CACHE_CTL_END);
     }
 }
 
@@ -680,7 +680,7 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
      * Start the cache cleanup thread after permanently dropping privileges.
      */
 #define VERIFY_DICT_OPEN_FLAGS (DICT_FLAG_DUP_REPLACE | DICT_FLAG_SYNC_UPDATE \
-	    | DICT_FLAG_OPEN_LOCK)
+	    | DICT_FLAG_OPEN_LOCK | DICT_FLAG_UTF8_REQUEST)
 
     saved_mask = umask(022);
     verify_map =
@@ -724,11 +724,11 @@ int     main(int argc, char **argv)
     MAIL_VERSION_STAMP_ALLOCATE;
 
     multi_server_main(argc, argv, verify_service,
-		      MAIL_SERVER_STR_TABLE, str_table,
-		      MAIL_SERVER_TIME_TABLE, time_table,
-		      MAIL_SERVER_PRE_INIT, pre_jail_init,
-		      MAIL_SERVER_POST_INIT, post_jail_init,
-		      MAIL_SERVER_SOLITARY,
-		      MAIL_SERVER_EXIT, verify_dump,
+		      CA_MAIL_SERVER_STR_TABLE(str_table),
+		      CA_MAIL_SERVER_TIME_TABLE(time_table),
+		      CA_MAIL_SERVER_PRE_INIT(pre_jail_init),
+		      CA_MAIL_SERVER_POST_INIT(post_jail_init),
+		      CA_MAIL_SERVER_SOLITARY,
+		      CA_MAIL_SERVER_EXIT(verify_dump),
 		      0);
 }
